/*
 * Copyright [2022] [https://www.xiaonuo.vip]
 *
 * Snowy采用APACHE LICENSE 2.0开源协议，您在使用过程中，需要注意以下几点：
 *
 * 1.请不要删除和修改根目录下的LICENSE文件。
 * 2.请不要删除和修改Snowy源码头部的版权声明。
 * 3.本项目代码可免费商业使用，商业使用请保留源码和相关描述文件的项目出处，作者声明等。
 * 4.分发源码时候，请注明软件出处 https://www.xiaonuo.vip
 * 5.不可二次分发开源参与同类竞品，如有想法可联系团队xiaonuobase@qq.com商议合作。
 * 6.若您的项目无法满足以上几点，需要更多功能代码，获取Snowy商业授权许可，请在官网购买授权，地址为 https://www.xiaonuo.vip
 */
package vip.xiaonuo.biz.modular.bizsettle.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollStreamUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.google.common.collect.Lists;
import jakarta.annotation.Resource;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import vip.xiaonuo.biz.modular.bizauditlog.entity.BizAuditLog;
import vip.xiaonuo.biz.modular.bizauditlog.service.BizAuditLogService;
import vip.xiaonuo.biz.modular.bizconfig.entity.BizConfig;
import vip.xiaonuo.biz.modular.bizconfig.service.BizConfigService;
import vip.xiaonuo.biz.modular.bizorder.entity.BizOrder;
import vip.xiaonuo.biz.modular.bizorder.param.BizOrderPageParam;
import vip.xiaonuo.biz.modular.bizorder.service.BizOrderService;
import vip.xiaonuo.biz.modular.bizsettle.param.*;
import vip.xiaonuo.biz.modular.bizsettleorder.entity.BizSettleOrder;
import vip.xiaonuo.biz.modular.bizsettleorder.service.BizSettleOrderService;
import vip.xiaonuo.common.enums.CommonSortOrderEnum;
import vip.xiaonuo.common.exception.CommonException;
import vip.xiaonuo.common.page.CommonPageRequest;
import vip.xiaonuo.biz.modular.bizsettle.entity.BizSettle;
import vip.xiaonuo.biz.modular.bizsettle.mapper.BizSettleMapper;
import vip.xiaonuo.biz.modular.bizsettle.service.BizSettleService;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.concurrent.locks.ReentrantLock;

/**
 * 运费结算Service接口实现类
 *
 * @author xiaofan
 * @date  2025/06/12 09:54
 **/
@Service
public class BizSettleServiceImpl extends ServiceImpl<BizSettleMapper, BizSettle> implements BizSettleService {
    private final ReentrantLock lock = new ReentrantLock();

    @Resource
    private BizSettleOrderService bizSettleOrderService;
    @Resource
    private BizOrderService bizOrderService;
    @Resource
    private BizConfigService bizConfigService;
    @Resource
    private BizAuditLogService bizAuditLogService;

    @Override
    public Page<BizSettle> page(BizSettlePageParam bizSettlePageParam) {
        QueryWrapper<BizSettle> queryWrapper = new QueryWrapper<BizSettle>().checkSqlInjection();
        if(ObjectUtil.isNotEmpty(bizSettlePageParam.getSettleNo())){
            queryWrapper.lambda().like(BizSettle::getSettleNo,bizSettlePageParam.getSettleNo());
        }
        if(ObjectUtil.isNotEmpty(bizSettlePageParam.getSettleType())){
            queryWrapper.lambda().eq(BizSettle::getSettleType,bizSettlePageParam.getSettleType());
        }
        if(ObjectUtil.isNotEmpty(bizSettlePageParam.getStartCreateTime()) && ObjectUtil.isNotEmpty(bizSettlePageParam.getEndCreateTime())){
            String format = DateUtil.format(new Date(), "yyyy-MM-dd");
            queryWrapper.lambda().between(BizSettle::getCreateTime,format+" 00:00:00",format+" 23:59:59");
        }
        if(ObjectUtil.isNotEmpty(bizSettlePageParam.getSettleStatus())){
            queryWrapper.lambda().eq(BizSettle::getSettleStatus,bizSettlePageParam.getSettleStatus());
        }
        Page<BizSettle> page = this.page(CommonPageRequest.defaultPage(), queryWrapper);
        return page;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(BizSettleAddParam bizSettleAddParam) {

        if(ObjectUtil.isNotEmpty(bizSettleAddParam.getSettleAccount())){
            BizConfig bizConfig = bizConfigService.getOne(new QueryWrapper<BizConfig>().lambda().last("limit 1"));
            if(bizSettleAddParam.getSettleAccount().compareTo(bizConfig.getSettleAccount()) < 0){
                throw new CommonException("结算总金额低于结算基础金额，不可计算！");
            }
        }

        BizSettle bizSettle = BeanUtil.toBean(bizSettleAddParam, BizSettle.class);
        String settleNo = getNumber();
        bizSettle.setSettleNo(settleNo);
        this.save(bizSettle);

        if(ObjectUtil.isNotEmpty(bizSettleAddParam.getOrderList())){
            for(BizOrder bizOrder : bizSettleAddParam.getOrderList()){
                //新增数据关联表
                BizSettleOrder bizSettleOrder = new BizSettleOrder();
                bizSettleOrder.setSettleId(bizSettle.getId());
                bizSettleOrder.setOrderId(bizOrder.getId());
                bizSettleOrderService.save(bizSettleOrder);

                UpdateWrapper<BizOrder> updateWrapper = new UpdateWrapper<>();
                //修改订单状态
                updateWrapper.set("order_status","9").eq("id",bizOrder.getId());
                bizOrderService.update(updateWrapper);
            }
        }
    }

    public String getNumber(){
        lock.lock();
        try{
            String format = new SimpleDateFormat("yyyyMMdd").format(new Date());
            String time = new SimpleDateFormat("yyyy-MM-dd").format(new Date());
            long count = this.count(new QueryWrapper<BizSettle>().lambda().between(BizSettle::getCreateTime, time + " 00:00:00", time + " 23:59:59"));
            String settleNo = "GSD" + format + generateCode((int)count);
            return settleNo;
        }finally {
            lock.unlock(); // 释放锁
        }

    }

    public static String generateCode(Integer counter) {
        counter++;
        return String.format("%03d", counter);
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void edit(BizSettleEditParam bizSettleEditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleEditParam.getId());
        BeanUtil.copyProperties(bizSettleEditParam, bizSettle);
        this.updateById(bizSettle);

        //删除之前关联信息
        List<BizSettleOrder> list = bizSettleOrderService.list(new QueryWrapper<BizSettleOrder>().lambda().
                eq(BizSettleOrder::getSettleId, bizSettle.getId()));
        //修改订单状态
        for(BizSettleOrder bizSettleOrder: list){
            BizOrder bizOrder = bizOrderService.getById(bizSettleOrder.getOrderId());
            if(ObjectUtil.isNotNull(bizOrder)){
                UpdateWrapper<BizOrder> updateWrapper = new UpdateWrapper<>();
                //修改订单状态
                updateWrapper.set("order_status","7").eq("id",bizOrder.getId());
                bizOrderService.update(updateWrapper);
            }
        }
        //删除关联信息表
        bizSettleOrderService.removeByIds(CollStreamUtil.toList(list,BizSettleOrder::getId));

        //新增新的关联信息
        if(ObjectUtil.isNotEmpty(bizSettleEditParam.getOrderList())){
            for(BizOrder bizOrder : bizSettleEditParam.getOrderList()){
                //新增数据关联表
                BizSettleOrder bizSettleOrder = new BizSettleOrder();
                bizSettleOrder.setSettleId(bizSettle.getId());
                bizSettleOrder.setOrderId(bizOrder.getId());
                bizSettleOrderService.save(bizSettleOrder);

                UpdateWrapper<BizOrder> updateWrapper = new UpdateWrapper<>();
                //修改订单状态
                updateWrapper.set("order_status","9").eq("id",bizOrder.getId());
                bizOrderService.update(updateWrapper);
            }
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void delete(List<BizSettleIdParam> bizSettleIdParamList) {
        // 执行删除
        //this.removeByIds(CollStreamUtil.toList(bizSettleIdParamList, BizSettleIdParam::getId));

        for(BizSettleIdParam bizSettleIdParam : bizSettleIdParamList){
            //删除关联表信息
            List<BizSettleOrder> list = bizSettleOrderService.list(new QueryWrapper<BizSettleOrder>().lambda().
                    eq(BizSettleOrder::getSettleId, bizSettleIdParam.getId()));
            //修改订单状态
            for(BizSettleOrder bizSettleOrder: list){
                BizOrder bizOrder = bizOrderService.getById(bizSettleOrder.getOrderId());
                if(ObjectUtil.isNotNull(bizOrder)){
                    UpdateWrapper<BizOrder> updateWrapper = new UpdateWrapper<>();
                    //修改订单状态
                    updateWrapper.set("order_status","7").eq("id",bizOrder.getId());
                    bizOrderService.update(updateWrapper);
                }
            }
            //删除关联信息表
            bizSettleOrderService.removeByIds(CollStreamUtil.toList(list,BizSettleOrder::getId));


            //删除运费结算信息
            this.removeById(bizSettleIdParam.getId());
        }
    }

    @Override
    public BizSettle detail(BizSettleIdParam bizSettleIdParam) {
        return this.queryEntity(bizSettleIdParam.getId());
    }

    @Override
    public BizSettle queryEntity(String id) {
        BizSettle bizSettle = this.getById(id);
        if(ObjectUtil.isEmpty(bizSettle)) {
            throw new CommonException("运费结算不存在，id值为：{}", id);
        }
        return bizSettle;
    }

    @Override
    public BizSettle detailOrder(BizSettleIdParam bizSettleIdParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleIdParam.getId());
        List<BizSettleOrder> list = bizSettleOrderService.list(new QueryWrapper<BizSettleOrder>().lambda().
                eq(BizSettleOrder::getSettleId, bizSettleIdParam.getId()));
        List<BizOrder> orderList = Lists.newArrayList();
        for(BizSettleOrder bizSettleOrder : list){
            BizOrderPageParam bizOrderPageParam = new BizOrderPageParam();
            bizOrderPageParam.setOrderId(bizSettleOrder.getOrderId());
            BizOrder detailOrder = bizOrderService.getDetailOrder(bizOrderPageParam);
            if(ObjectUtil.isNotNull(detailOrder)){
                orderList.add(detailOrder);
            }
        }
        bizSettle.setOrderList(orderList);
        return bizSettle;
    }

    @Override
    public void submit(BizSettleEditParam bizSettleEditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleEditParam.getId());
        bizSettle.setSettleStatus("2");
        this.updateById(bizSettle);
    }

    /**管理员审批*/
    @Transactional
    @Override
    public void auditAdmin(BizSettleAuditParam bizSettleAuditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleAuditParam.getId());
        String status = "2";
        String auditStatus = "";
        if(StringUtils.equals(bizSettleAuditParam.getAuditFlag(),"true")){
            status="4";
            auditStatus = "1";
        }else{
            status = "3";
            auditStatus = "2";
        }
        //修改状态
        UpdateWrapper<BizSettle> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("settle_status",status).eq("id",bizSettle.getId());
        this.update(updateWrapper);

        //插入审核记录
        BizAuditLog bizAuditLog = new BizAuditLog();
        bizAuditLog.setType("YFJS");
        bizAuditLog.setDataId(bizSettle.getId());
        bizAuditLog.setStatus(auditStatus);
        bizAuditLog.setRemark(bizSettleAuditParam.getSettleReason());
        bizAuditLogService.save(bizAuditLog);
    }

    /**财务审批*/
    @Transactional
    @Override
    public void auditFinance(BizSettleAuditParam bizSettleAuditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleAuditParam.getId());
        String status = "";
        String auditStatus = "";
        if(StringUtils.equals(bizSettleAuditParam.getAuditFlag(),"true")){
            status="5";
            auditStatus = "1";
        }else{
            status = "3";
            auditStatus = "2";
        }
        //修改状态
        UpdateWrapper<BizSettle> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("settle_status",status).eq("id",bizSettle.getId());
        this.update(updateWrapper);

        //插入审核记录
        BizAuditLog bizAuditLog = new BizAuditLog();
        bizAuditLog.setType("YFJS");
        bizAuditLog.setDataId(bizSettle.getId());
        bizAuditLog.setStatus(auditStatus);
        bizAuditLog.setRemark(bizSettleAuditParam.getSettleReason());
        bizAuditLogService.save(bizAuditLog);
    }

    @Override
    public void uploadInvoice(BizSettleEditParam bizSettleEditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleEditParam.getId());
        if(ObjectUtil.isNotEmpty(bizSettleEditParam.getFileNameList())){
            StringBuffer fileNameBuffer = new StringBuffer();
            for(String fileName : bizSettleEditParam.getFileNameList()){
                fileNameBuffer.append(fileName+",");
            }
            bizSettle.setFileInvoiceName(fileNameBuffer.substring(0,fileNameBuffer.length()-1));
        }
        if(ObjectUtil.isNotEmpty(bizSettleEditParam.getFilePathList())){
            StringBuffer filePathBuffer = new StringBuffer();
            for(String filePath : bizSettleEditParam.getFilePathList()){
                filePathBuffer.append(filePath+",");
            }
            bizSettle.setFileInvoicePath(filePathBuffer.substring(0,filePathBuffer.length()-1));
        }
        bizSettle.setSettleStatus("6");
        this.updateById(bizSettle);
    }

    @Override
    public void uploadVoucher(BizSettleEditParam bizSettleEditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleEditParam.getId());
        if(ObjectUtil.isNotEmpty(bizSettleEditParam.getFileNameList())){
            StringBuffer fileNameBuffer = new StringBuffer();
            for(String fileName : bizSettleEditParam.getFileNameList()){
                fileNameBuffer.append(fileName+",");
            }
            bizSettle.setFileVoucherName(fileNameBuffer.substring(0,fileNameBuffer.length()-1));
        }
        if(ObjectUtil.isNotEmpty(bizSettleEditParam.getFilePathList())){
            StringBuffer filePathBuffer = new StringBuffer();
            for(String filePath : bizSettleEditParam.getFilePathList()){
                filePathBuffer.append(filePath+",");
            }
            bizSettle.setFileVoucherPath(filePathBuffer.substring(0,filePathBuffer.length()-1));
        }
        bizSettle.setSettleStatus("7");
        this.updateById(bizSettle);
    }

    @Override
    public void auditReject(BizSettleAuditParam bizSettleAuditParam) {
        BizSettle bizSettle = this.queryEntity(bizSettleAuditParam.getId());

        //修改状态
        UpdateWrapper<BizSettle> updateWrapper = new UpdateWrapper<>();
        updateWrapper.set("settle_status","5").eq("id",bizSettle.getId());
        this.update(updateWrapper);

        //插入审核记录
        BizAuditLog bizAuditLog = new BizAuditLog();
        bizAuditLog.setType("YFJS");
        bizAuditLog.setDataId(bizSettle.getId());
        bizAuditLog.setStatus("2");
        bizAuditLog.setRemark(bizSettleAuditParam.getSettleReason());
        bizAuditLogService.save(bizAuditLog);
    }
}
